<?php

namespace Concrete\Package\SamlAuthentication\Authentication\Saml;

use Concrete\Core\Authentication\Type\ExternalConcrete5\ServiceFactory;
use Concrete\Core\Authentication\Type\OAuth\OAuth2\GenericOauth2TypeController;
use Concrete\Core\Config\Repository\Repository;
use Concrete\Core\Url\Resolver\Manager\ResolverManagerInterface;
use Concrete\Core\User\Group\GroupList;
use Concrete\Core\User\User;
use InvalidArgumentException;
use League\Url\Url;
use UserInfo;

//require_once dirname(__FILE__). DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR. 'autoload.php';
require_once dirname(__FILE__). DIRECTORY_SEPARATOR . 'toolkit_loader.php';
require_once  dirname(__FILE__). DIRECTORY_SEPARATOR . 'settings.php';


class Controller extends GenericOauth2TypeController
{

    /** @var \Concrete\Core\Authentication\Type\ExternalConcrete5\ServiceFactory */
    protected $factory;

    /** @var \Concrete\Core\Url\Resolver\Manager\ResolverManagerInterface */
    protected $urlResolver;

    /** @var \Concrete\Core\Config\Repository\Repository */
    protected $config;



	protected $samlsettings;
		
    public function __construct(
        \Concrete\Core\Authentication\AuthenticationType $type = null,
        ServiceFactory $factory,
        ResolverManagerInterface $urlResolver,
        Repository $config)
    {
        parent::__construct($type);
        $this->factory = $factory;
        $this->urlResolver = $urlResolver;
        $this->config = $config;
		
		$spBaseUrl = $this->urlResolver->resolve(['/']);
		
		$this->samlsettings = array (
			'sp' => array (
					'entityId' => $spBaseUrl.'/sso/',
					'assertionConsumerService' => array (
						'url' => $spBaseUrl.'/login',
					),
					'singleLogoutService' => array (
						'url' => $spBaseUrl.'/index.php/login?action=logout',
					),
					'NameIDFormat' => 'urn:oasis:names:tc:SAML:1.1:nameid-format:unspecified',
				),
				'idp' => array (
					'entityId' => $this->config->get('auth.saml_auth.appid'),
					'singleSignOnService' => array (
						'url' => $this->config->get('auth.saml_auth.url'),
					),
					'singleLogoutService' => array (
						'url' => '',
					),
					'x509cert' => $this->config->get('auth.saml_auth.secret'),
				),

			
			'email_attribute' => 'email',	
			'first_name_attribute' => 'fname',
			'last_name_attribute' => 'lname',
			'role_attribute' => 'Role',
							
		);
		
		
    }

    /**
     * Get the ID of the group to enter upon registration
     * This method grabs the registration group ID out of the auth config group
     *
     * @return int
     */
    public function registrationGroupID()
    {
        return (int) $this->config->get('auth.saml_auth.registration.group');
    }

    /**
     * Determine whether this type supports automatically registering users
     * This method grabs this configuration from the auth config group
     *
     * @return bool
     */
    public function supportsRegistration()
    {
        return (bool) $this->config->get('auth.saml_auth.registration.enabled', false);
    }

    /**
     * Build and return this authentication type's icon HTML
     *
     * @return string
     */
    public function getAuthenticationTypeIconHTML()
    {
        $svgData = file_get_contents(DIR_BASE_CORE . '/images/authentication/community/concrete.svg');
        $publicSrc = '/concrete/images/authentication/community/concrete.svg';

        return "<div class='ccm-concrete-authentication-type-svg' data-src='{$publicSrc}'>{$svgData}</div>";
    }

    public function getHandle()
    {
        return 'saml';
    }

    /**
     * Get the service object associated with this authentication type
     * This method uses the oauth/factory/service object to create our service if one is not set
     *
     * @return \Concrete\Core\Authentication\Type\ExternalConcrete5\ExternalConcrete5Service
     */
    public function getService()
    {
        if (!$this->service) {
            /** @var \OAuth\ServiceFactory $serviceFactory */
            $serviceFactory = $this->app->make('oauth/factory/service');
            $this->service = $this->factory->createService($serviceFactory);
        }

        return $this->service;
    }

    /**
     * Save data for this authentication type
     * This method is called when the type_form.php submits. It stores client details and configuration for connecting
     *
     * @param array|\Traversable $args
     */
    public function saveAuthenticationType($args)
    {
        $passedUrl = trim($args['url']);

        if ($passedUrl) {
            try {
                $url = Url::createFromUrl($passedUrl);

                if (!(string)$url->getScheme() || !(string)$url->getHost()) {
                    throw new InvalidArgumentException('No scheme or host provided.');
                }

            } catch (\Exception $e) {
                throw new InvalidArgumentException('Invalid URL.');
            }
        }

        $passedName = trim($args['displayName']);
        if (!$passedName) {
            throw new InvalidArgumentException('Invalid display name');
        }
        $this->authenticationType->setAuthenticationTypeName($passedName);

        $config = $this->app->make(Repository::class);
        $config->save('auth.saml_auth.url', $args['url']);
        $config->save('auth.saml_auth.appid', $args['apikey']);
        $config->save('auth.saml_auth.secret', $args['ssocertificate']);
        $config->save('auth.saml_auth.registration.enabled', (bool) $args['registration_enabled']);
        $config->save('auth.saml_auth.registration.group', intval($args['registration_group'], 10));
    }

    /**
     * Controller method for type_form
     * This method is called just before rendering type_form.php, use it to set data for that template
     */
    public function edit()
    {
        $config = $this->app->make(Repository::class);
        $this->set('form', $this->app->make('helper/form'));
        $this->set('data', $config->get('auth.saml_auth', []));
        $this->set('redirectUri', $this->samlsettings['sp']['assertionConsumerService']['url']);
		$this->set('spEntityId', $this->samlsettings['sp']['entityId']);

        $list = $this->app->make(GroupList::class);
        $this->set('groups', $list->getResults());
    }

    /**
     * Controller method for form
     * This method is called just before form.php is rendered, use it to set data for that template
     */
    public function form()
    {
        $this->setData();
    }

    /**
     * Controller method for the hook template
     * This method is called before hook.php is rendered, use it to set data for that template
     */
    public function hook()
    {
        $this->setData();
    }

    /**
     * Controller method for the hooked template
     * This method gets called before hooked.php is rendered, use it to set data for that template
     */
    public function hooked()
    {
        $this->setData();
    }

    /**
     * Method for setting general data for all views
     */
    private function setData()
    {
		
	
		//$auth = new \OneLogin\Saml2\Auth($this->samlsettings); 
		//$auth->login(); 
		
		$authnRequest = new \OneLogin\Saml2\AuthnRequest(new \OneLogin\Saml2\Settings($this->samlsettings), false, false, null);
		$authUrl = $this->samlsettings['idp']['singleSignOnService']['url'] . '?SAMLRequest=' .  urlencode($authnRequest->getRequest());
		
        $data = $this->config->get('auth.saml_auth', '');
        //$authUrl = $this->urlResolver->resolve(['/ccm/system/authentication/oauth2/saml_auth/attempt_auth']);
        $attachUrl = $this->urlResolver->resolve(['/ccm/system/authentication/oauth2/saml_auth/attempt_attach']);
        $baseUrl = $this->urlResolver->resolve(['/']);
        $path = $baseUrl->getPath();
        $path->remove('index.php');
        $name = trim((string) array_get($data, 'name', t('SSO')));

        $this->set('data', $data);
        $this->set('authUrl', $authUrl);
        $this->set('attachUrl', $attachUrl);
        $this->set('baseUrl', $baseUrl);
        $this->set('assetBase', $baseUrl->setPath($path));
        $this->set('name', $name);
        $this->set('user', $this->app->make(User::class));
		
		if(isset($_POST['SAMLResponse'])) {
		
			$email = $fname = $lname = "";

			$auth = new \OneLogin\Saml2\Response(new \OneLogin\Saml2\Settings($this->samlsettings), $_POST['SAMLResponse']); 
			if(!$auth->isValid()) {
				echo "SSO request not valid. Please try again.<br>";
			} else {
				
				$username = $auth->getNameId();
				
				/*$email = $auth->getAttribute($settings['email_attribute']);
				if(is_array($email) && sizeof($email)>0)
					$email = $email[0];
				$fname = $auth->getAttribute($settings['first_name_attribute']);
				if(is_array($fname) && sizeof($fname)>0)
					$fname = $fname[0];
				$lname = $auth->getAttribute($settings['last_name_attribute']);
				if(is_array($lname) && sizeof($lname)>0)
					$lname = $lname[0];

				if(!empty($fname)) {
					$fullname = $fname . " " .$lname;
				} else
					$fullname = $username;
				*/
				
				$ui = UserInfo::getByEmail($username);
				
				if($ui ==null) 
					$ui = UserInfo::getByUserName($username);
				
				if($ui ==null) { 
					try {
						$user = $this->createNewUser($username, $username);
					} catch (Exception $e) {
						$user = \User::loginByUserID($ui->getUserID());
					}
				} else {
					$user = \User::loginByUserID($ui->getUserID());
				}

				header('Location: ./');
				exit;
			}
		}
		
    }
	
	
	public function createNewUser($username, $email, $first_name='', $last_name = '')
    {

        $data = array();
        $data['uName'] = $username;
        $data['uPassword'] = \Illuminate\Support\Str::random(256);
        $data['uEmail'] = $email;
        $data['uIsValidated'] = 1;

        $user_info = \UserInfo::add($data);

        if (!$user_info) {
            throw new Exception('Unable to create new account.');
        }

        if ($group_id = intval($this->registrationGroupID(), 10)) {
            $group = \Group::getByID($group_id);
            if ($group && is_object($group) && !$group->isError()) {
                $user = \User::getByUserID($user_info->getUserID());
                $user->enterGroup($group);
            }
        }

        $key = \UserAttributeKey::getByHandle('first_name');
        if ($key) {
            $user_info->setAttribute($key, $first_name);
        }

        $key = \UserAttributeKey::getByHandle('last_name');
        if ($key) {
            $user_info->setAttribute($key, $last_name);
        }

        $user = \User::loginByUserID($user_info->getUserID());

        return $user;
    }
	
}
